package com.broadlink.flutter.blsdk_plugin;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import cn.com.broadlink.sdk.data.controller.BLDNADevice;

class DeviceManager {
    private HashMap<String, BLDNADevice> probedDevice;
    private HashMap<String, Long> probedTime;

    //设备超时时间，如果TimeOut内没有收到心跳则认为已经不在局域网
    private final int TimeOut = 9000;

    public DeviceManager() {
        probedDevice = new HashMap<>();
        probedTime = new HashMap<>();
    }

    public synchronized void addProbedBlDevice(BLDNADevice bldnaDevice) {
        probedDevice.put(bldnaDevice.getDid(), bldnaDevice);
        probedTime.put(bldnaDevice.getDid(), System.currentTimeMillis());
    }

    public synchronized List<BLDNADevice> getProbedBldevice() {
        List<BLDNADevice> bldnaDeviceList = new ArrayList<>();
        Iterator<Map.Entry<String, Long>> iterator = probedTime.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, Long> entry = iterator.next();
            String key = entry.getKey();
            long probedTime = entry.getValue();
            if ((System.currentTimeMillis() - probedTime) > TimeOut) {
                //超时不在局域网
                probedDevice.remove(key);
                iterator.remove();
            } else {
                BLDNADevice bldnaDevice = probedDevice.get(key);
                if (bldnaDevice != null) {
                    BLDNADevice copyDevice = bldnaDevice.copy();
                    bldnaDeviceList.add(copyDevice);
                }
            }
        }
        return bldnaDeviceList;
    }

    public void clearProbedDevice() {
        probedDevice.clear();
        probedTime.clear();
    }
}
