//
//  BLAppSDKManager.m
//  blsdk_plugin
//
//  Created by 王振 on 2021/8/24.
//

#import "BLAppSDKManager.h"
#import <BLLetCore/BLLetCore.h>
#import <BLLetBase/BLLetBase.h>
#import <BLLetFamily/BLLetFamily.h>
#import <BLLetAccount/BLLetAccount.h>


@interface BLAppSDKManager ()<BLControllerDelegate>
@property (strong,nonatomic)NSMutableArray <BLDNADevice *>*localDeviceArray;
@end

@implementation BLAppSDKManager

+(instancetype)sharedManager {
    static dispatch_once_t onceToken;
    static BLAppSDKManager *manager = nil;
    dispatch_once(&onceToken, ^{
        manager = [[self alloc] init];
    });
    return manager;
}
-(NSMutableArray<BLDNADevice *> *)localDeviceArray {
    if (_localDeviceArray == nil) {
        _localDeviceArray = [NSMutableArray array];
    }
    return _localDeviceArray;
}
-(BOOL)blAppSDKInitWithInfo:(NSDictionary *)info {

    BLConfigParam *config = [BLConfigParam sharedConfigParam];
    config.appServiceEnable = 1;                // 2019年之后的APP版本都使用集群方式
    config.appServiceHost = [info objectForKey:@""];       // 配置APP连接集群地址
    config.sdkFileSavePath = [self sharedPath];

    NSMutableDictionary *connetServiceDict = [[NSMutableDictionary alloc] init];
    if ([info.allKeys containsObject:@"device_http_server"]) {
        [connetServiceDict setValue:[info objectForKey:@"device_http_server"] forKey:@"http"];
    }
    if ([info.allKeys containsObject:@"device_tcp_server"]){
        [connetServiceDict setValue:[info objectForKey:@"device_tcp_server"] forKey:@"tcp"];
    }
    if ([info.allKeys containsObject:@"companyid"]) {
        [connetServiceDict setValue:[info objectForKey:@"companyid"] forKey:@"companyid"];
    }
    config.deviceConnectServiceInfo = [connetServiceDict copy];     //配置下发固件连接域名


    config.controllerScriptDownloadVersion = 1;                     // 2019年之后的APP都使用新的脚本下载服务
    config.controllerSendCount = 1;                                 // 设备控制重试次数
    config.controllerLocalTimeout = 3000;                           // 设备控制局域网超时时间
    config.controllerRemoteTimeout = 5000;                          // 设备控制远程超时时间
    config.controllerNetMode = BL_NET_DEFAULT;                      // 设备控制模式
    config.controllerSupportProtocol = BLDeviceSupportDefault;      // 设备扫描支持模式
    config.isLoginMutexMode = NO; //账号互斥

    //SDK 初始化
    if ([info.allKeys containsObject:@"license"]) {
        BLLet *bllet = [BLLet sharedLetWithLicense:[info objectForKey:@"license"]];
    //#ifdef DEBUG
    //    bllet.debugLog = BL_LEVEL_ALL;
    //    [bllet.controller setSDKRawDebugLevel:BL_LEVEL_DEBUG];
    //#else
        bllet.debugLog = BL_LEVEL_NONE;
        [bllet.controller setSDKRawDebugLevel:BL_LEVEL_NONE];
    //#endif
        [bllet.controller startProbe:3000];
//        [BLAccount sharedAccount];
        //设置代理
        [BLLet sharedLet].controller.delegate = self;
    }else {
        return NO;
    }
    return YES;
}
-(void)startPorb {
    [[BLLet sharedLet].controller startProbe:3000];
}
-(void)stopProb {
    [[BLLet sharedLet].controller stopProbe];
}
-(NSArray *)getProbedDeviceCache {
    NSMutableArray *arr = [NSMutableArray array];
    for (BLDNADevice *device in self.localDeviceArray) {
        NSDictionary *dic = @{
            @"gateway_id":device.pDid?device.pDid:@"",
            @"did":device.did?device.did:@"",
            @"pid":device.pid?device.pid:@"",
            @"name":device.name?device.name:@"",
            @"key":device.controlKey?device.controlKey:@""
        };
        [arr addObject:dic];
    }
    return [arr copy];
}

-(void)controlDeviceWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    NSString *did = [info objectForKey:@"did"];
    NSString *pdid = [info objectForKey:@"gateway_id"];
    NSArray *keys = [[info objectForKey:@"command"] allKeys];
    NSArray *values = [[info objectForKey:@"command"] allValues];
    NSInteger sendCount = [[[info objectForKey:@"control_params"] objectForKey:@"CONTROLLER_LOCAL_TIMEOUT"] integerValue];
    sendCount = sendCount > 0 ? sendCount:1;
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        NSMutableDictionary *sendDict = [[NSMutableDictionary alloc] init];
        [sendDict setObject:@"set" forKey:@"act"];
        [sendDict setObject:did forKey:@"did"];
        [sendDict setObject:keys forKey:@"params"];
        NSMutableArray *valueArray = [[NSMutableArray alloc] init];
        for (int i = 0; i < values.count; i++) {
            NSArray *valueDict = @[@{
                                        @"val" :values[i],
                                        @"idx" : @1
                                        }];
            [valueArray addObject:valueDict];
        }
        [sendDict setObject:valueArray forKey:@"vals"];
        NSString *controlString = [self p_toJsonString:sendDict];
        NSString *controlResult = [[BLLet sharedLet].controller dnaControl:pdid.length > 0 ?pdid:did subDevDid:pdid.length > 0 ? did:nil dataStr:controlString command:@"dev_ctrl" scriptPath:nil sendcount:sendCount];
        NSDictionary *backResult = [self p_toDictory:controlResult];
        if ([[backResult objectForKey:@"status"] integerValue] == 0) {
            dispatch_async(dispatch_get_main_queue(), ^{
                success(backResult);
            });
        } else{
            dispatch_async(dispatch_get_main_queue(), ^{
                failure(backResult);
            });
        }
    });
}
-(void)getDeviceStatusWithInfo:(NSDictionary *)info success:(successIdWithBlock)success failure:(failureIdWithBlock)failure{
    NSString *did = [info objectForKey:@"did"];
    NSString *pdid = [info objectForKey:@"gateway_id"];
    NSArray *keys = [[info objectForKey:@"command"] allKeys];
    NSArray *values = [[info objectForKey:@"command"] allValues];
    NSInteger sendCount = [[[info objectForKey:@"control_params"] objectForKey:@"CONTROLLER_LOCAL_TIMEOUT"] integerValue];
    sendCount = sendCount > 0 ? sendCount:1;
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        NSMutableDictionary *sendDict = [[NSMutableDictionary alloc] init];
        [sendDict setObject:@"get" forKey:@"act"];
        [sendDict setObject:did forKey:@"did"];
        [sendDict setObject:keys forKey:@"params"];
        NSMutableArray *valueArray = [[NSMutableArray alloc] init];
        for (int i = 0; i < values.count; i++) {
            NSArray *valueDict = @[@{
                                        @"val" :values[i],
                                        @"idx" : @1
                                        }];
            [valueArray addObject:valueDict];
        }
        [sendDict setObject:valueArray forKey:@"vals"];
        NSString *controlString = [self p_toJsonString:sendDict];
        NSString *controlResult = [[BLLet sharedLet].controller dnaControl:pdid.length > 0 ?pdid:did subDevDid:pdid.length > 0 ? did:nil dataStr:controlString command:@"dev_ctrl" scriptPath:nil sendcount:sendCount];
        NSDictionary *backResult = [self p_toDictory:controlResult];
        if ([[backResult objectForKey:@"status"] integerValue] == 0) {
            dispatch_async(dispatch_get_main_queue(), ^{
                success(backResult);
            });
        } else{
            dispatch_async(dispatch_get_main_queue(), ^{
                failure(backResult);
            });
        }
    });
}

- (NSString *)p_toJsonString:(NSDictionary *)dic {
    if (dic == nil) {
        return @"";
    }
    NSError *error;
    NSData *data = [NSJSONSerialization dataWithJSONObject:dic options:0 error:&error];
    if (error) {
        NSLog(@"%@", error.localizedDescription);
        return nil;
    }

    return [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
}

- (NSDictionary *)p_toDictory:(id)json {
    if (!json || json == (id)kCFNull) return nil;
    NSDictionary *dic = nil;
    NSData *jsonData = nil;
    if ([json isKindOfClass:[NSDictionary class]]) {
        dic = json;
    } else if ([json isKindOfClass:[NSString class]]) {
        jsonData = [(NSString *)json dataUsingEncoding : NSUTF8StringEncoding];
    } else if ([json isKindOfClass:[NSData class]]) {
        jsonData = json;
    }
    if (jsonData) {
        dic = [NSJSONSerialization JSONObjectWithData:jsonData options:kNilOptions error:NULL];
        if (![dic isKindOfClass:[NSDictionary class]]) dic = nil;
    }
    return dic;
}
#pragma mark - BLControllerDelegate
- (Boolean)filterDevice:(BLDNADevice *)device {
    return NO;
}

- (Boolean)shouldAdd:(BLDNADevice *)device {
    //本地搜索到设备不会自动添加到SDK内
    return NO;
}

- (void)onDeviceUpdate:(BLDNADevice *)device isNewDevice:(Boolean)isNewDevice {
    BOOL isContain = NO;
    for (BLDNADevice *tempDevice in self.localDeviceArray) {
        if ([tempDevice.did isEqualToString:device.did]) {
            isContain = YES;
            tempDevice.controlId = device.controlId;
            tempDevice.controlKey = device.controlKey;
            tempDevice.lanaddr = device.lanaddr;
            tempDevice.newConfig = device.newConfig;
            break;
        }
    }
    if (!isContain) {
        [self.localDeviceArray addObject:device];
    }
}

- (NSString *)sharedPath {
    BLConfigParam *config = [BLConfigParam sharedConfigParam];
    return config.sdkFileSavePath;
}

@end
