
import 'dart:async';

import 'package:flutter/services.dart';

class BlsdkPlugin {
  static const MethodChannel _channel =
      const MethodChannel('blsdk_plugin');

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  ///初始化broadlink sdk
  //{
  // 	"local_timeout": "",
  // 	"remote_timeout": "",
  // 	"device_tcp_server": "",
  // 	"device_http_server": "",
  // 	"lid": "",
  // 	"license": "",
  // 	"channel_id": "",
  // 	"app_host": ""
  // }
  ///
  static Future<bool?> initBLSDK(String initData) async {
    final bool? result = await _channel.invokeMethod('init_blsdk'
        ,<String,dynamic>{'data':initData});
    return result;
  }

  ///发送控制
  //参数
  // {
  //       "gateway_id": "",
  //       "did": "",
  //       "command": {
  //
  //       },
  //       "control_params": {}
  // }
  //返回数据
  // {
  // "status": "",
  // "msg": "",
  // "data": {
  // "pwr": 1,
  // "brightness": 100,
  // "lightrate": 50
  // }
  // }
  ///
  static Future<String?> controlCommand(String command) async {
    final String? result = await _channel.invokeMethod('control_command'
        ,<String,dynamic>{'data':command});
    return result;
  }
  ///发送查询命令
  //参数
  // {
  //       "gateway_id": "",
  //       "did": "",
  //       "control_params": {}
  // }
  //返回数据
  // {
  // "status": "",
  // "msg": "",
  // "data": {
  // "pwr": 1,
  // "brightness": 100,
  // "lightrate": 50
  // }
  // }
  static Future<String?> queryCommand(String command) async {
    final String? result = await _channel.invokeMethod('query_command'
        ,<String,dynamic>{'data':command});
    return result;
  }

  //打开循环发现局域网设备
  static Future<void> startProbe(int probeInterval) async {
    await _channel.invokeMethod('start_probe'
        ,<String,dynamic>{'probe_interval':probeInterval});
  }

  //停止发现局域网设备
  static void stopProbe() async {
    await _channel.invokeMethod('stop_probe');
  }
  ///
  // [
  // 	{
  // 		"gateway_id": "",
  // 		"did": "",
  // 		"pid": "",
  // 		"name": "",
  // 		"key": ""
  // 	}
  // ]
  ///
  //获取发现的设备列缓存列表
  static Future<String?> getProbedDevice() async{
    final String? deviceListStr = await _channel.invokeMethod('get_probed_device_cache');
    return deviceListStr;
  }

  //清除发现的设备缓存列表
  static Future<void> clearProbedDevice() async{
    await _channel.invokeMethod('clear_probed_device_cache');
  }
}
